@@ -1,19 +1,28 @@
 class TestCase(object):
     def __init__(self, desc):
         self.__desc = desc
-        self.params = {}
         self.logger = None
         self.timeout = -1
         self.retry = 0
         self.__sub_desc = ""
         self.__sub_params = []
         self.test_params = None
+        self.execute_func = None
+        self.prepare_func = None
+        self.teardown_func = None
     def prepare(self):
-        pass
+        if self.prepare_func:
+            self.prepare_func()
     def execute(self):
-        raise Exception("Not implemented.")
+        if not self.execute_func:
+            raise Exception("Not implemented.")
+        if self.__sub_params == []:
+            self.execute_func()
+        else:
+            self.execute_func(self.test_params)
     def teardown(self):
-        pass
+        if self.teardown_func:
+            self.teardown_func()
     def set_sub_params(self, params):
         if not isinstance(params, list):
             raise Exception("Subtest parameters needs to be of type list.")
@@ -37,8 +46,6 @@ class TestCase(object):
             suffix = str(p)
         self.__sub_desc = self.__desc + " - " + suffix
         self.test_params = p
-    def set_params(self, params):
-        self.params = params
     def is_active(self, args):
        active = vars(args)[self.__desc.replace("-","_").replace(" ", "_").lower()]
        return active
